# an example plugin for mpdtunes that builds a trayicon.

import gtk
from egg import trayicon
import sys


def mpd_pause(obj, mpd):
    mpd.pause()

def mpd_next(obj, mpd):
    mpd.next()

def mpd_prev(obj, mpd):
    mpd.prev()

def mpd_play(obj, mpd):
    mpd.play()

def mpd_quit(obj, mainWindow):
    mainWindow.destroy()
    
class Trayicon:

    # called when we load the plugin. we get passed the mainWindow, plugin menu
    # item as well as a connection to mpd.
    def _init(self, data):
        self.xml, self.pluginMenu, self.pympd, self.base_dir, self.parser = data
        self.mainWindow = self.xml.get_widget("mainWindow")
        self.data = data
        self.buildIcon()
        self.buildMenu()
        self.buildPluginMenu(self.pluginMenu)

    def _conf(self, hasConf=False):
        return False

    # update plugin, data is of type mpdclient.Status
    def _spin(self, data, songChanged=False):
        status = data
        stateStr = status.state
        if stateStr == 'play':
            self.image.set_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_LARGE_TOOLBAR)
            if songChanged:
                song = self.pympd.getCurrentSong()
            else:
                try:
                    song = self.song
                except Exception, e:
                    song = self.pympd.getCurrentSong()
            self.tooltip.set_tip(self.icon, "Now Playing:\n%s - %s"%(song.artist, song.title))
            self.song = song
        elif stateStr == 'pause':
            self.image.set_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_LARGE_TOOLBAR)
            self.tooltip.set_tip(self.icon, "pympd\n(Paused)")
        self.stateStr = stateStr
        self.tooltip.enable()

    # called on plugin unloaded
    def _unload(self):
        self.pluginMenuEntry.destroy()
        self.icon.destroy()
        self.menu.destroy()
        self.mainWindow.present()
        return

    # callback for click on icon
    def icon_clicked(self, obj, event):
        if event.button == 1:
            if not self.mainWindow.get_property('visible'):
                self.mainWindow.show()
            else:
                self.mainWindow.hide()
        elif event.button == 3:
            self.menu.popup(None, None, None, event.button, event.time)
            pass # bring up right click

    # callback for menu item in pluginMenu
    def toggleIcon(self, obj):
        if self.icon.get_property('visible'):
            self.icon.hide()
        else:
            self.icon.show()
              
    # Init Functions
    # build the menu for pluginMenu. 
    # NOTE: you build a menuItem() and append it to the plugin menu passed in from pympd
    def buildPluginMenu(self, pluginMenu):
        self.pluginMenuEntry = gtk.MenuItem('Trayicon')
        pluginSubMenu = gtk.Menu()
        pluginSubMenu.set_title('Trayicon')
        menuEntry = gtk.CheckMenuItem('Show Icon in Tray')
        menuEntry.activate()
        menuEntry.connect('activate', self.toggleIcon)
        pluginSubMenu.append(menuEntry)
        self.pluginMenuEntry.set_submenu(pluginSubMenu)
        self.pluginMenuEntry.show_all()
        pluginMenu.append(self.pluginMenuEntry)
    
    # build the right click menu for the icon.
    def buildMenu(self):
        self.menu = gtk.Menu()
        
        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
        pause = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PAUSE)
        next = gtk.ImageMenuItem(gtk.STOCK_MEDIA_NEXT)
        prev = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PREVIOUS)
        separator = gtk.MenuItem()
        quit = gtk.ImageMenuItem(gtk.STOCK_QUIT)
        
        self.menu.append(play)
        self.menu.append(pause)
        self.menu.append(prev)
        self.menu.append(next)
        self.menu.append(separator)
        self.menu.append(quit)

        play.connect('activate', mpd_play, self.pympd)
        pause.connect('activate', mpd_pause, self.pympd)
        next.connect('activate', mpd_next, self.pympd)
        prev.connect('activate', mpd_prev, self.pympd)
        quit.connect('activate', mpd_quit, self.mainWindow)

        self.menu.show_all()

    def buildIcon(self):
        self.icon = trayicon.TrayIcon("mpdTunes")
        self.stateStr = None
        self.image = gtk.Image()
        self.image.set_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.tooltip = gtk.Tooltips()
        self.tooltip.set_tip(self.icon, "mpd Tunes")
        self.tooltip.enable()
        eventBox = gtk.EventBox()
        eventBox.add(self.image)
        eventBox.connect('button-press-event', self.icon_clicked)
        self.icon.add(eventBox)
        self.icon.show_all()
